##########################################################################
# xmlexcel - convert XML into a comma-separated CVS file
#            suitable for importing as a spreadsheet
#
# Tony Garland - contact@SpiritAndTruth.org
#
# Anyone may freely redistribute and use this program.
#
##########################################################################
import os.path
import sys
import re
import string
import codecs
from xml.sax import make_parser
from xml.sax.handler import feature_namespaces
from xml.sax import saxutils

class docHandler( saxutils.DefaultHandler ) :
    def __init__( self, outFile ) :
        self.m_outFile = outFile
        self.m_inRecord = 0
        self.m_fieldContent = {}
        self.m_fieldNames = []
        self.m_fieldName = ''
        self.m_inElement = 0
        self.m_records = []
        self.m_firstElementName = ''
        self.m_excelFieldSeparator = '\t'

    def startElement( self, name, attrs ) :
        #self.m_outFile.write( '%s\n' % name )
        if not self.m_firstElementName : 
            self.m_firstElementName = name
        if name == 'record' : 
            self.m_inRecord = 1
            self.m_fieldContent = {}
            self.m_fieldNames = []
        if self.m_inRecord :
            self.m_fieldNames.append( name )
            if self.m_fieldContent.has_key ( name ) :
                self.m_fieldContent[ name ] += ' '
            else:
                self.m_fieldContent[ name ] = ''
        self.m_inElement = 1

    def characters( self, ch ):
        if self.m_inRecord and self.m_inElement :
            #ch = re.sub( '([^\t\r\n]+)(\s+)$', r'\1', ch )
            ch = re.sub( '[\r\n]', r'', ch )
            ch = re.sub( '^\s+$', r'', ch )
            ch = re.sub( '\s+', r' ', ch )
            self.m_fieldContent[ self.m_fieldNames[ -1 ] ] += ch

    def endElement( self, name ):
        #self.m_outFile.write( '\t%s\n' % name )
        if name == 'record' : 
            self.m_inRecord = 0
            prevName = ''
            infoByName = {}
            for name in self.m_fieldNames :
                if name == prevName : continue
                prevName = name
                info = self.m_fieldContent[ name ]
                if info :
                    infoByName[ name ] = info
            self.m_records.append( infoByName )
        self.m_inElement = 0
        if name == self.m_firstElementName : 
            self.printRecords()

    def printRecords( self ) :
        # Build list of all names found in all records.
        allnames = []
        for infoByName in self.m_records :
            for name in infoByName.keys() :
                if not allnames.count( name ) :
                    allnames.append( name )

        # Print heading line.
        self.m_outFile.write( string.join( allnames, self.m_excelFieldSeparator ) )
        self.m_outFile.write( '\n' )

        # For each record, print record.
        for infoByName in self.m_records :
            fields = []
            for name in allnames :
                if infoByName.has_key( name ) :
                    info = self.excelFormat( infoByName[ name ] )
                    fields.append( info )
                else :
                    fields.append( '-' )
            self.m_outFile.write( string.join( fields,  self.m_excelFieldSeparator ) )
            self.m_outFile.write( '\n' )

    def excelFormat( self, text ) :
        # Convert text ready to output in Excel text format.
        # Fields can't contain the separator character itself.
        return re.sub( self.m_excelFieldSeparator, ' ', text )

if __name__ == '__main__':

    # Usage.
    if len( sys.argv ) < 2 :
        print 'Usage: %s file1 [file2 . . . ]' % sys.argv[ 0 ]
        sys.exit( 1 )

    # Get Unicode reader and writer.
    ( unic_encode, unic_decode, unic_reader, unic_writer ) = codecs.lookup( 'utf-8' )

    # Create a parser
    parser = make_parser()

    # Tell the parser we are not interested in XML namespaces
    parser.setFeature(feature_namespaces, 0)

    # For each file passed as input.
    for file in sys.argv[ 1 : ] : 
        print file

        # Create corresponding output file.
        inFileName = file
        outFileName, ext = os.path.splitext( file )
        outFileName += '.txt'
        inFile = outFile = None
        try :

            inFile = open( inFileName, 'r' )
            inFileReader = unic_reader( inFile )
            outFile = open( outFileName, 'w' )
            outFileWriter = unic_writer( outFile )
            print '%s -> %s' % ( inFileName, outFileName )

            # Create the handler
            handler = docHandler( outFileWriter )

            # Tell the parser to use our handler
            parser.setContentHandler( handler )

            # Parse the file
            parser.parse( inFile )

        finally :
            if inFile: inFile.close()
            if outFile: outFile.close()
